@@ -18,11 +18,12 @@ use strict;
 use Spreadsheet::BIFFwriter;
 use Spreadsheet::Worksheet;
 use Spreadsheet::OLEwriter;
+use Spreadsheet::Format;
 
 use vars qw($VERSION @ISA);
 @ISA = qw(Spreadsheet::BIFFwriter Exporter);
 
-$VERSION = '0.02';
+$VERSION = '0.03';
 
 ######################################################################
 #
@@ -38,14 +39,18 @@ sub new {
 
     $self->{_store_in_memory}= $_[1] || 0;
     $self->{_OLEwriter}      = Spreadsheet::OLEwriter->new($filename);
-    $self->{_activesheet}    = 0;
-    $self->{_firstsheet}     = 0;
+    $self->{_active_sheet}   = 0;
+    $self->{_first_sheet}    = 0;
     $self->{_fileclosed}     = 0;
     $self->{_biffsize}       = 0;
     $self->{_sheetname}      = "Sheet";
+    $self->{_tmp_worksheet}  = Spreadsheet::Worksheet->new('', 0, 0);
+    $self->{_tmp_format}     = Spreadsheet::Format->new();
     $self->{_worksheets}     = [];
+    $self->{_formats}        = [];
 
     bless $self, $class;
+    #$self->addformat(); # Used to write the default XFs
     return $self;
 }
 
@@ -62,7 +67,7 @@ sub close {
     my $self = shift;
 
     return if $self->{_fileclosed}; # Prevent calling close() twice
-    
+
     $self->_store_workbook();
     $self->{_OLEwriter}->close();
     $self->{_fileclosed} = 1;
@@ -121,8 +126,6 @@ sub addworksheet {
     my @init_data = (
                         $name,
                         $index,
-                        \$self->{_activesheet},
-                        \$self->{_firstsheet},
                         $self->{_store_in_memory},
                     );
 
@@ -131,6 +134,23 @@ sub addworksheet {
     return $worksheet;
 }
 
+######################################################################
+#
+# addformat()
+#
+# Add a new format to the Excel workbook. This adds an XF record and
+# a FONT record. TODO: add a FORMAT record.
+#
+sub addformat {
+
+    my $self      = shift;
+
+    my $format = Spreadsheet::Format->new();
+    push @{$self->{_formats}}, $format;
+    return $format;
+}
+
+
 
 ######################################################################
 #
@@ -229,7 +249,7 @@ sub _store_workbook {
     foreach my $sheet (@{$self->{_worksheets}}) {
         $sheet->_close();
     }
- 
+
     # Add Workbook globals
     $self->_store_bof(0x0005);
     $self->_store_window1();
@@ -260,125 +280,30 @@ sub _store_workbook {
 }
 
 
-######################################################################
-#
-# BIFF RECORDS
-#
-
-
-######################################################################
-#
-# _store_window1()
-#
-# Write Excel BIFF WINDOW1 record.
-#
-sub _store_window1 {
-
-    my $self      = shift;
-    my $name      = 0x003D; # Record identifier
-    my $length    = 0x0012; # Number of bytes to follow
-
-    my $xWn       = 0x0000; # Horizontal position of window
-    my $yWn       = 0x0069; # Vertical position of window
-    my $dxWn      = 0x339F; # Width of window
-    my $dyWn      = 0x5D1B; # Height of window
-
-    my $grbit     = 0x0038; # Option flags
-    my $ctabsel   = 0x0001; # Number of workbook tabs selected
-    my $wTabRatio = 0x0258; # Tab to scrollbar ratio
-
-    my $itabFirst = $self->{_firstsheet};  # 1st displayed worksheet
-    my $itabCur   = $self->{_activesheet}; # Selected worksheet
-
-    my $header    = pack("vv",        $name, $length);
-    my $data      = pack("vvvvvvvvv", $xWn, $yWn, $dxWn, $dyWn,
-                                      $grbit,
-                                      $itabCur, $itabFirst,
-                                      $ctabsel, $wTabRatio);
-
-    $self->_append($header, $data);
-}
-
-
-######################################################################
-#
-# _store_font($fontname)
-#
-# Write Excel BIFF FONT record.
-#
-sub _store_font {
-
-    my $self      = shift;
-    my $font      = $_[0];
-    my $cch       = length($font);
-
-    my $name      = 0x0031;        # Record identifier
-    my $length    = 0x000F + $cch; # Bytes to follow
-
-    my $dyHeight  = 0x00C8; # Height of font (1/20 of a point)
-    my $grbit     = 0x0000; # Font attributes
-    my $icv       = 0x7FFF; # Index to color palette
-    my $bls       = 0x0190; # Bold style
-    my $sss       = 0x0000; # Superscript/subscript
-    my $uls       = 0x00;   # Underline
-    my $bFamily   = 0x00;   # Font family
-    my $bCharSet  = 0x00;   # Character set
-    my $reserved  = 0x00;   # Reserved
-
-    my $header    = pack("vv",         $name, $length);
-    my $data      = pack("vvvvvCCCCC", $dyHeight, $grbit, $icv, $bls,
-                                       $sss, $uls, $bFamily,
-                                       $bCharSet, $reserved, $cch);
-
-    $self->_append($header, $data, $font);
-}
-
-
 ######################################################################
 #
 # _store_all_fonts()
 #
-# Write all FONT records.
-#
+# Store the Excel FONT records. TODO: At present a FONT record is
+# created for each XF record. Duplicates should be removed.
 sub _store_all_fonts {
 
-    my $self    = shift;
+    my $self   = shift;
 
-    $self->_store_font('Arial');
-    $self->_store_font('Arial');
-    $self->_store_font('Arial');
-    $self->_store_font('Arial');
-    $self->_store_font('Arial');
-}
+    # _tmp_format is added by new() and not by the user.
+    # We use  this to write the default XF's
+    my $format = $self->{_tmp_format};
+    my $font   = $format->get_font();
 
+    for (1..4){
+        $self->_append($font);
+    }
 
-######################################################################
-#
-# _store_xf()
-#
-# Write Excel BIFF XF record.
-#
-sub _store_xf {
-
-    my $self      = shift;
-    my $name      = 0x00E0; # Record identifier
-    my $length    = 0x0010; # Number of bytes to follow
-
-    my $ifnt      = 0x0000; # Index to FONT record
-    my $ifmt      = 0x0000; # Index to FORMAT record
-    my $style     = $_[0];  # Style and other options
-    my $align     = 0x0020; # Alignment
-    my $icv       = 0x20C0; # Color palette and other options
-    my $fill      = 0x0000; # Fill and border line style
-    my $brd_line  = 0x0000; # Border line style and color
-    my $brd_color = 0x0000; # Border color
-
-    my $header    = pack("vv",       $name, $length);
-    my $data      = pack("vvvvvvvv", $ifnt, $ifmt, $style, $align,
-                                     $icv, $fill,
-                                     $brd_line, $brd_color);
-
-    $self->_append($header, $data);
+    # User defined fonts
+    foreach $format (@{$self->{_formats}}) {
+        $font = $format->get_font();
+        $self->_append($font);
+    }
 }
 
 
@@ -392,35 +317,28 @@ sub _store_all_xfs {
 
     my $self    = shift;
 
+    # _tmp_format is added by new() and not by the user.
+    # We use  this to write the default XF's
+    my $format = $self->{_tmp_format};
+    my $xf;
+
+    # Set the font attribute to the default value
+    $format->{_font_index} = 0;
+
     for (0..14) {
-        $self->_store_xf(0xFFF5); # Cell XF
+        $xf = $format->get_xf(0xFFF5); # Style XF
+        $self->_append($xf);
     }
 
-    $self->_store_xf(0x0001);     # Style XF
-}
+    $xf = $format->get_xf(0x0001);     # Cell XF
+    $self->_append($xf);
 
 
-######################################################################
-#
-# _store_style()
-#
-# Write Excel BIFF STYLE records.
-#
-sub _store_style {
-
-    my $self      = shift;
-
-    my $name      = 0x0093; # Record identifier
-    my $length    = 0x0004; # Bytes to follow
-
-    my $ixfe      = 0x0000; # Index to style XF
-    my $BuiltIn   = 0x00;   # Built-in style
-    my $iLevel    = 0x00;   # Outline style level
-
-    my $header    = pack("vv",  $name, $length);
-    my $data      = pack("vCC", $ixfe, $BuiltIn, $iLevel);
-
-    $self->_append($header, $data);
+    # User defined formats
+    foreach $format (@{$self->{_formats}}) {
+        $xf = $format->get_xf(0x0001);
+        $self->_append($xf);
+    }
 }
 
 
@@ -438,6 +356,51 @@ sub _store_all_styles {
 }
 
 
+######################################################################
+#
+# BIFF RECORDS
+#
+
+
+######################################################################
+#
+# _store_window1()
+#
+# Write Excel BIFF WINDOW1 record.
+#
+sub _store_window1 {
+
+    my $self      = shift;
+    my $name      = 0x003D;  # Record identifier
+    my $length    = 0x0012;  # Number of bytes to follow
+
+    my $xWn       = 0x0000;  # Horizontal position of window
+    my $yWn       = 0x0000;  # Vertical position of window
+    my $dxWn      = 0x25BC;  # Width of window
+    my $dyWn      = 0x1572;  # Height of window
+
+    my $grbit     = 0x0038;  # Option flags
+    my $ctabsel   = 0x0001;  # Number of workbook tabs selected
+    my $wTabRatio = 0x0258;  # Tab to scrollbar ratio
+
+
+    my $worksheet = $self->{_tmp_worksheet};
+    my $first     = $worksheet->get_first_sheet();
+    my $active    = $worksheet->get_active_sheet();
+
+    my $itabFirst = $first;  # 1st displayed worksheet
+    my $itabCur   = $active; # Selected worksheet
+
+    my $header    = pack("vv",        $name, $length);
+    my $data      = pack("vvvvvvvvv", $xWn, $yWn, $dxWn, $dyWn,
+                                      $grbit,
+                                      $itabCur, $itabFirst,
+                                      $ctabsel, $wTabRatio);
+
+    $self->_append($header, $data);
+}
+
+
 ######################################################################
 #
 # _store_boundsheet()
@@ -462,6 +425,29 @@ sub _store_boundsheet {
     $self->_append($header, $data, $sheetname)
 }
 
+######################################################################
+#
+# _store_style()
+#
+# Write Excel BIFF STYLE records.
+#
+sub _store_style {
+
+    my $self      = shift;
+
+    my $name      = 0x0093; # Record identifier
+    my $length    = 0x0004; # Bytes to follow
+
+    my $ixfe      = 0x0000; # Index to style XF
+    my $BuiltIn   = 0x00;   # Built-in style
+    my $iLevel    = 0x00;   # Outline style level
+
+    my $header    = pack("vv",  $name, $length);
+    my $data      = pack("vCC", $ixfe, $BuiltIn, $iLevel);
+
+    $self->_append($header, $data);
+}
+
 
 1;
 
